// nymph sandbox shovel
//
// by stagas
//
// modified version of:
// shovel.js - Do the heavy lifting in this sandbox
// Gianni Chiappetta - gf3.ca - 2010

/* ------------------------------ INIT ------------------------------ */
var util = require( 'util' )
  , fs = require('fs')
  , path = require('path')
  , crypto = require('crypto')
  , code
  , console
  , fsop = 0
  , maxfsop = 5
  , result
  , sandbox
  , Script
  , stdin

var args = JSON.parse(process.env.ARGS)

if (!(Script = process.binding('evals').NodeScript))
  Script = process.binding('evals').Script

/* ------------------------------ Sandbox ------------------------------ */
// Sandbox methods
args.dirname = path.join(args.dirname, 'store')
try {
  fs.mkdirSync(args.dirname, 0755)
} catch(e) {}
function Store(name) {
  var datafile = path.normalize(path.join(args.dirname, name.replace(/\.\./igm, '')))
  var data
  try {
    data = JSON.parse(fs.readFileSync(datafile, 'utf8'))
  } catch(e) {
    data = {}
  }
  var proxy = {
    get: function(key) {
      if (null == key || fsop > maxfsop) return
      fsop++
      if (data.propertyIsEnumerable(key)) {
        return data[key]
      }
      return 'key "' + key + '" not found'    
    }
  , set: function(key, val) {
      if (null == key || null == val || fsop > maxfsop) return
      fsop++
      data[key] = val && val.toString() || ''
      try {
        fs.writeFileSync(datafile, JSON.stringify(data), 'utf8')
        return 'set "' + key + '" to "' + val + '"'
      } catch(e) {
        return 'key "' + key + '" not written'
      }  
    }
  , del: function(key) {
      delete data[key]
      try {
        fs.writeFileSync(datafile, JSON.stringify(data), 'utf8')
        return 'deleted "' + key + '"'
      } catch(e) {
        return 'key "' + key + '" not deleted'
      }
    }
  , clear: function() {
      data = {}
      try {
        fs.writeFileSync(datafile, '{}', 'utf8')
        return 'cleared'
      } catch(e) {
        return 'not cleared'
      }
    }
  , get list() {
      return Object.keys(data)
    }
  , forEach: function(fn) {
      Object.keys(data).sort().forEach(function(key) {
        fn.call(data[key], key, data[key].toString())
      })
    }
  }
  return {
    get: function(key) {
      return proxy.get(key)
    }
  , set: function(key, val) {
      return proxy.set(key, val)
    }
  , del: function(key) {
      return proxy.del(key)
    }
  , clear: function() {
      return proxy.clear()
    }
  , get list() {
      return proxy.list
    }
  , forEach: function(fn) {
      return proxy.forEach(fn)
    }
  }
}

var thisStore = Store(args.name)

console = []
sandbox = {
  console: {
    log: function() {
      var i, l, msg
      for ( i = 0, l = arguments.length; i < l; i++ ) {
        msg = util.inspect(arguments[i])
        if (msg && msg.length > 2 && msg[0] === "'" && msg[msg.length - 1] === "'") {
          msg = msg.substr(1, msg.length - 2)
        }
        console.push(msg)
      }
    }
  }
}
sandbox.say = sandbox.print = sandbox.console.log
sandbox.get = thisStore.get
sandbox.set = thisStore.set
sandbox.del = thisStore.del
sandbox.list = thisStore.list
sandbox.clear = thisStore.clear
sandbox.load = function(name) {
  return Store(name)
}

if (args.args) {
  sandbox.a = args.args[0]
  sandbox.b = args.args[1]
  sandbox.c = args.args[2]
  sandbox.d = args.args[3]
  sandbox.e = args.args[4]
  sandbox.f = args.args[5]
  sandbox.args = args.args
}

// Get code
code = ''
stdin = process.stdin
stdin.resume()
stdin.setEncoding('utf8')
stdin.on('data', function(data) {
  code += data
})
stdin.on('end', run)

// Run code
function run() {
  result = (function() {
    try {
      return Script.runInNewContext('with(this){' + this.toString() + '}', sandbox)
    }
    catch (e) {
      return e.name + ': ' + e.message
    }
  }).call(code)
  
  process.stdout.on('drain', function() {
    process.exit(0)
  })

  var msg = util.inspect(result)
  if (msg && msg.length > 2 && msg[0] === "'" && msg[msg.length - 1] === "'") {
    msg = msg.substr(1, msg.length - 2)
  }
  process.stdout.write(JSON.stringify({ result: msg, console: console }))
}
